@@ -15,11 +15,15 @@ import json
 import logging
 import os
 import signal
+from datetime import datetime as dt
+from datetime import timedelta as td
+from queue import Empty
+from threading import Lock
 from typing import Callable, Dict, List, Optional, Tuple
 
 from .command import Command
 from .const import ATTR_DEVICES, ATTR_ORPHANS, NUL_DEVICE_ID, __dev_mode__
-from .devices import DEVICE_CLASSES, Device
+from .devices import Device, create_device
 from .message import Message, process_msg
 from .packet import set_pkt_logging
 from .protocol import create_msg_stack
@@ -27,20 +31,17 @@ from .schema import (
     ALLOW_LIST,
     BLOCK_LIST,
     DEBUG_MODE,
-    DISABLE_DISCOVERY,
-    DISABLE_SENDING,
     DONT_CREATE_MESSAGES,
-    GLOBAL_CONFIG_SCHEMA,
     INPUT_FILE,
-    PACKET_LOG,
-    REDUCE_PROCESSING,
     load_config_schema,
     load_system_schema,
 )
-from .systems import SYSTEM_CLASSES, System, SystemBase
+from .systems import System, create_system
 from .transport import POLLER_TASK, create_pkt_stack
 from .version import __version__  # noqa: F401
 
+I_, RQ, RP, W_ = " I", "RQ", "RP", " W"
+
 DEV_MODE = __dev_mode__ and False
 VERSION = __version__
 
@@ -69,39 +70,34 @@ class Gateway:
         self.serial_port = serial_port
         self._input_file = kwargs.pop(INPUT_FILE, None)
 
-        (self.config, self._include, self._exclude) = load_config_schema(
-            serial_port, self._input_file, **GLOBAL_CONFIG_SCHEMA(kwargs)
+        (self.config, schema, self._include, self._exclude) = load_config_schema(
+            self.serial_port, self._input_file, **kwargs
         )
 
         set_pkt_logging(
-            cc_stdout=self.config[REDUCE_PROCESSING] >= DONT_CREATE_MESSAGES,
-            **self.config[PACKET_LOG],
+            cc_stdout=self.config.reduce_processing >= DONT_CREATE_MESSAGES,
+            **self.config.packet_log,
         )
 
         self.pkt_protocol, self.pkt_transport = None, None
         self.msg_protocol, self.msg_transport = None, None
 
-        # if self.config[REDUCE_PROCESSING] >= DONT_CREATE_MESSAGES:
-        #     return
-
-        if self.config[REDUCE_PROCESSING] < DONT_CREATE_MESSAGES:
+        if self.config.reduce_processing < DONT_CREATE_MESSAGES:
             self.msg_protocol, self.msg_transport = self.create_client(process_msg)
 
-        # self._buffer = deque()
-        # self._sched_zone = None
-        # self._sched_lock = Lock()
+        self._state_lock = Lock()
+        self._state_params = None
 
-        # if self.config[REDUCE_PROCESSING] > 0:
+        # if self.config.reduce_processing > 0:
         self.rfg = None
         self.evo = None
-        self.systems: List[SystemBase] = []
+        self.systems: List[System] = []
         self.system_by_id: Dict = {}
         self.devices: List[Device] = []
         self.device_by_id: Dict = {}
 
         self._prev_msg = None
 
-        schema = {k: v for k, v in kwargs.items() if k not in self.config}
         self.known_devices = load_system_schema(self, **schema)
 
         self._setup_event_handlers()
@@ -151,6 +147,7 @@ class Gateway:
             raise RuntimeError("Unsupported OS for this module: %s", os.name)
 
     async def start(self) -> None:
+        _LOGGER.debug("ENGINE: Starting...")
         if self.serial_port:  # source of packets is a serial port
             self.pkt_protocol, self.pkt_transport = create_pkt_stack(
                 self,
@@ -180,76 +177,51 @@ class Gateway:
     def _get_device(self, dev_addr, ctl_addr=None, domain_id=None, **kwargs) -> Device:
         """Return a device (will create it if required).
 
+        NB: a device can be safely considered bound to a controller only if the
+        controller says it is.
+
         Can also set a controller/system (will create as required). If a controller is
         provided, can also set the domain_id as one of: zone_idx, FF (controllers), FC
         (heater_relay), HW (DHW sensor, relay), or None (unknown, TBA).
         """
 
-        def create_system(ctl, profile=None) -> SystemBase:
-            # assert ctl.id not in self.system_by_id, f"Dup. sys_id: {ctl.id}"
+        if dev_addr.type in ("18", "--") or dev_addr.id in (NUL_DEVICE_ID, "01:000001"):
+            return  # not valid device types/real devices
 
-            if profile is None:
-                profile = "programmer" if dev_addr.type == "23" else "evohome"
+        if self._include and dev_addr.id not in self._include:
+            _LOGGER.warning(
+                f"Found a non-allowed device_id: {dev_addr.id}"
+                f" (consider addding it to the {ALLOW_LIST})"
+            )
 
-            system = SYSTEM_CLASSES.get(profile, System)(self, ctl)
-            if self.evo is None:
-                self.evo = system
-
-            if not self.config[DISABLE_DISCOVERY]:
-                system._discover()  # discover_flag=DISCOVER_ALL)
-            return system
-
-        def create_device(dev_addr) -> Device:  # TODO: Optional[Device]
-            # assert dev_addr.id not in self.device_by_id, f"Dup. dev_id: {dev_addr.id}"
-
-            if self._include and dev_addr.id not in self._include:
-                _LOGGER.warning(
-                    f"Creating a non-allowed device_id: {dev_addr.id}"
-                    f" (consider addding it to the {ALLOW_LIST})"
-                )
-
-            elif dev_addr.id in self._exclude:
-                _LOGGER.warning(
-                    f"Creating a blocked device_id: {dev_addr.id}"
-                    f" (consider removing it from the {BLOCK_LIST})"
-                )
-
-            # else:
-            device = DEVICE_CLASSES.get(dev_addr.type, Device)(self, dev_addr)
-
-            # if isinstance(device, Controller):
-            # if device._is_controller:
-            # if dev_addr.type in SYSTEM_CLASSES:
-            # if domain_id == "FF"
-            if dev_addr.type in ("01", "23"):
-                device._evo = create_system(device, profile=kwargs.get("profile"))
-
-            if not self.config[DISABLE_DISCOVERY]:
-                device._discover()  # discover_flag=DISCOVER_ALL)
-            return device
+        if dev_addr.id in self._exclude:
+            _LOGGER.warning(
+                f"Found a blocked device_id: {dev_addr.id}"
+                f" (consider removing it from the {BLOCK_LIST})"
+            )
 
         if ctl_addr is not None:
             ctl = self.device_by_id.get(ctl_addr.id)
             if ctl is None:
                 ctl = self._get_device(ctl_addr, domain_id="FF", **kwargs)
 
-        if dev_addr.type in ("18", "--") or dev_addr.id in (NUL_DEVICE_ID, "01:000001"):
-            return  # not valid device types/real devices
-
         dev = self.device_by_id.get(dev_addr.id)
         if dev is None:  # TODO: take into account device filter?
-            dev = create_device(dev_addr)
+            dev = create_device(self, dev_addr)
+
+        if dev._is_controller and dev._evo is None:
+            dev._evo = create_system(self, dev, profile=kwargs.get("profile"))
 
         if not self.rfg and dev.type == "18":
             self.rfg = dev
 
-        # update the existing device with any metadata
+        # update the existing device with any metadata TODO: this is messy
         if ctl_addr and ctl:
             dev._set_ctl(ctl)
         if domain_id in ("F9", "FA", "FC", "FF"):
             dev._domain_id = domain_id
         elif domain_id is not None and ctl_addr and ctl:
-            dev._set_zone(ctl._evo._get_zone(domain_id))
+            dev._set_parent(ctl._evo._get_zone(domain_id))
 
         return dev
 
@@ -263,49 +235,80 @@ class Gateway:
         gwy.device_by_id = {}
         gwy.devices = []
 
-    def _pause_engine(self) -> Tuple[Callable, bool, bool]:
+    def _pause_engine(self) -> None:
+        self._state_lock.acquire()
+        if self._state_params is not None:
+            self._state_lock.release()
+            raise RuntimeError("Unable to pause, the engine is already paused")
+
         callback = None
 
         if self.pkt_protocol:
             self.pkt_protocol.pause_writing()
             self.pkt_protocol._callback, callback = None, self.pkt_protocol._callback
 
-        self.config[DISABLE_DISCOVERY], discovery = True, self.config[DISABLE_DISCOVERY]
-        self.config[DISABLE_SENDING], sending = True, self.config[DISABLE_SENDING]
+        self.config.disable_discovery, discovery = True, self.config.disable_discovery
+        self.config.disable_sending, sending = True, self.config.disable_sending
 
-        return (callback, discovery, sending)
+        self._state_params = (callback, discovery, sending)
+        self._state_lock.release()
+
+    def _resume_engine(self) -> None:
+        self._state_lock.acquire()
+        if self._state_params is None:
+            self._state_lock.release()
+            raise RuntimeError("Unable to resume, the engine is not paused")
+
+        self._state_params, (callback, discovery, sending) = None, self._state_params
 
-    def _resume_engine(
-        self, callback: Callable, discovery: bool, sending: bool
-    ) -> None:
         if self.pkt_protocol:
             self.pkt_protocol._callback = callback  # self.msg_transport._pkt_receiver
             self.pkt_protocol.resume_writing()
 
-        self.config[DISABLE_DISCOVERY] = discovery
-        self.config[DISABLE_SENDING] = sending
+        self.config.disable_discovery = discovery
+        self.config.disable_sending = sending
+
+        # [
+        #     zone._discover(discover_flag=6)
+        #     for evo in self.systems
+        #     for zone in evo.zones
+        # ]
+        self._state_lock.release()
 
     def _get_state(self) -> Tuple[Dict, Dict]:
-        engine_state = self._pause_engine()
+        self._pause_engine()
+        _LOGGER.info("ENGINE: Saving state...")
 
-        msgs = {v.dtm: v for d in self.devices for v in d._msgs.values()}
+        # msgs = {v.dtm: v for d in self.devices for v in d._msgs.values()}
+        msgs = {
+            pkt.dtm: pkt
+            for device in self.devices
+            for verb in device._msgz.values()
+            for code in verb.values()
+            for pkt in code.values()
+        }
         for system in self.systems:
             msgs.update({v.dtm: v for v in system._msgs.values()})
             msgs.update({v.dtm: v for z in system.zones for v in z._msgs.values()})
+            # msgs.update({v.dtm: v for z in system._dhw for v in z._msgs.values()})
 
         pkts = {
             dtm.isoformat(sep="T", timespec="auto"): repr(msg)
             for dtm, msg in msgs.items()
-            if not msg.is_expired
+            if msg.verb in (I_, RP)
+            and not msg.is_expired
+            and msg.dtm >= dt.now() - td(days=7)  # HACK: ideally, wouldn't be any >7d
         }
 
         schema, pkts = self.schema, dict(sorted(pkts.items()))
 
-        self._resume_engine(*engine_state)
+        _LOGGER.info("ENGINE: Saved state")
+        self._resume_engine()
         return schema, pkts
 
     async def _set_state(self, schema: Dict, packets: Dict) -> None:
-        engine_state = self._pause_engine()
+        self._pause_engine()
+        _LOGGER.info("ENGINE: Restoring state...")
 
         self.known_devices = load_system_schema(self, **schema)  # keep old known_devs?
 
@@ -316,7 +319,15 @@ class Gateway:
         )
         await tmp_transport.get_extra_info(POLLER_TASK)
 
-        self._resume_engine(*engine_state)
+        while not self.msg_transport._que.empty():
+            try:
+                self.msg_transport._que.get_nowait()
+            except Empty:
+                continue
+            self.msg_transport._que.task_done()
+
+        _LOGGER.info("ENGINE: Restored state")
+        self._resume_engine()
 
     @property
     def schema(self) -> dict:
@@ -366,6 +377,18 @@ class Gateway:
         """
         if not self.msg_protocol:
             raise RuntimeError("there is no message protocol")
+
+        async def scheduler(period, *args, **kwargs):
+            while True:
+                await self.msg_protocol.send_data(*args, **kwargs)
+                await asyncio.sleep(period)  # drift is OK
+
+        periodic = kwargs.pop("period", None)  # a timedelta object
+        if periodic:
+            return self._loop.create_task(
+                scheduler(periodic.total_seconds(), cmd, **kwargs)
+            )
+
         return self._loop.create_task(
             self.msg_protocol.send_data(cmd, callback=callback, **kwargs)
         )
